/// Licensed Materials - Property of IBM 
/// Rational Team Concert
///  Copyright IBM Corporation 2010. All Rights Reserved. 
/// U.S. Government Users Restricted Rights -  Use, duplication or disclosure restricted by GSA DP  Schedule Contract with IBM Corp. 

using System;
using System.IO;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace JazzConsoleApplication
{
    class Program
    {
        private const string AUTH_MSG_KEY = "X-com-ibm-team-repository-web-auth-msg";
        private const string AUTH_REQUIRED_MSG = "authrequired";
        private const string BASIC_AUTHENTICATION_TYPE = "Basic";

        static void Main(string[] args)
        {
            string repository = "https://jazzserver.abc.def:9443/jazz/";
            string username = "username";
            string password = "password";

            //modify only if proxy requires authentication
            string proxyUsername = "proxyUsername";
            string proxyPassword = "proxyPassword";

            try
            {
                ServicePointManager.Expect100Continue = false;
                ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(Callback);

#if NET_45      // TLS 1.1/1.2 supported in .NET Framework 4.5 and later
                ServicePointManager.SecurityProtocol = ServicePointManager.SecurityProtocol | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
#endif

                SecurityContext context = SecurityContext.CreateSecurityContext(username, password, proxyUsername, proxyPassword);

                Console.WriteLine("Initializing repository...");
                HttpWebResponse initResponse = SendRequest(repository, "secure/service/com.ibm.team.repository.service.internal.webuiInitializer.IWebUIInitializerRestService/initializationData?", context);
                string authMsg = initResponse.Headers[AUTH_MSG_KEY];
                if (authMsg == AUTH_REQUIRED_MSG)
                {
                    Console.WriteLine("Performing Security Check...");
                    HttpWebResponse authResponse = SendRequest(repository, string.Format("j_security_check?j_username={0}&j_password={1}", username, password), context);
                    authMsg = authResponse.Headers[AUTH_MSG_KEY];
                    if (!string.IsNullOrEmpty(authMsg))
                    {
                        Console.WriteLine("Web Authentication Message: " + authMsg);
                        Exit();
                        return;
                    }
                }

                Console.WriteLine("Getting Project Areas...");
                SendRequest(repository, "secure/service/com.ibm.team.process.internal.common.service.IProcessRestService/allProjectAreas?", context);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }

            Exit();
        }

        static bool Callback(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            //trust all certificates
            return true;
        }

        class SecurityContext
        {
            internal static SecurityContext CreateSecurityContext(string username, string password, string proxyUsername, string proxyPassword)
            {
                NetworkCredential credential = new NetworkCredential(username, password);

                WebHeaderCollection authHeader = new WebHeaderCollection();
                byte[] credBuf = new UTF8Encoding().GetBytes(username + ":" + password);
                authHeader.Add(HttpRequestHeader.Authorization, BASIC_AUTHENTICATION_TYPE + " " + Convert.ToBase64String(credBuf));

                CookieContainer cookie = new CookieContainer();

                NetworkCredential proxyCredential = new NetworkCredential(proxyUsername, proxyPassword);

                return new SecurityContext(credential, authHeader, cookie, proxyCredential);
            }

            internal NetworkCredential Credential;
            internal WebHeaderCollection AuthHeader;
            internal CookieContainer Cookie;
            internal NetworkCredential ProxyCredential;

            SecurityContext(NetworkCredential credential, WebHeaderCollection authHeader, CookieContainer cookie, NetworkCredential proxyCredential)
            {
                this.Credential = credential;
                this.AuthHeader = authHeader;
                this.Cookie = cookie;
                this.ProxyCredential = proxyCredential;
            }
        }

        static HttpWebResponse SendRequest(string repository, string service, SecurityContext context)
        {
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(repository + service);
            request.AllowAutoRedirect = true;
            request.KeepAlive = true;
            request.Timeout = 480000;
            request.Credentials = context.Credential;
            request.Headers.Add(context.AuthHeader);
            request.CookieContainer = context.Cookie;
            Console.Write("Request Uri:--------------------------------------------------------------------");
            Console.WriteLine(request.RequestUri);
            Console.Write("Request Headers:----------------------------------------------------------------");
            Console.Write(request.Headers);

            IWebProxy proxy = WebRequest.DefaultWebProxy;
            
            Uri proxyUri = proxy.GetProxy(request.RequestUri);
            Console.Write("Proxy Uri:----------------------------------------------------------------------");
            Console.WriteLine(proxyUri);
            bool isBypassed = proxy.IsBypassed(request.RequestUri);
            Console.Write("Is Bypassed:--------------------------------------------------------------------");
            Console.WriteLine(isBypassed);

            if (!isBypassed)
                proxy.Credentials = context.ProxyCredential;

            HttpWebResponse response = (HttpWebResponse)request.GetResponse();
            Console.Write("Response Headers:---------------------------------------------------------------");
            Console.Write(response.Headers);
            Console.Write("Response:-----------------------------------------------------------------------");
            Console.WriteLine(ReadResponse(response));
            Console.WriteLine();
            return response;
        }

        static string ReadResponse(HttpWebResponse response)
        {
            StreamReader reader = new StreamReader(response.GetResponseStream());
            return reader.ReadToEnd();
        }

        static void Exit()
        {
            Console.Write("Press any key to exit...");
            Console.ReadKey();
        }
    }
}
